/*
 * Copyright (c) 2008,2018 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <sys/types.h>
#include <curses.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include <rmp/rmpLib.h>

void
showData(struct RMP_DEV_STR *dev, int col)
{
	struct RMP_DATA_STR *rmp = rmpGetData(dev);
	int i = 3;
	//pthread_mutex_lock(rmpGetMutex(dev));
	mvprintw(i++, col, "pitch angle: %d\n", rmp->pitch_angle);
	mvprintw(i++, col, "pitch rate:  %d\n", rmp->pitch_rate);
	mvprintw(i++, col, "roll angle:  %d\n", rmp->roll_angle);
	mvprintw(i++, col, "roll rate:   %d\n", rmp->roll_rate);
	mvprintw(i++, col, "yaw rate:    %d\n", rmp->yaw_rate);
	mvprintw(i++, col, "left wheel velocity:  %d\n", rmp->lw_velocity);
	mvprintw(i++, col, "right wheel velocity: %d\n", rmp->rw_velocity);
	mvprintw(i++, col, "servo frames: %d\n", rmp->servo_frames);
	mvprintw(i++, col, "integrated left wheel:  %d\n", rmp->integrated_left_wheel);
	mvprintw(i++, col, "integrated right wheel: %d\n", rmp->integrated_right_wheel);
	mvprintw(i++, col, "integrated fore/aft:    %d\n", rmp->integrated_fore_aft);
	mvprintw(i++, col, "integrated yaw:         %d\n", rmp->integrated_yaw);
	mvprintw(i++, col, "left torque:  %d\n", rmp->left_torque);
	mvprintw(i++, col, "right torque: %d\n", rmp->right_torque);
	mvprintw(i++, col, "operational mode:  %d\n", rmp->operational_mode);
	mvprintw(i++, col, "controller gain schedule: %d\n", 
	    rmp->controller_gain_schedule);
	mvprintw(i++, col, "ui_voltage:        %d\n", rmp->ui_voltage);
	mvprintw(i++, col, "powerbase voltage: %d\n", rmp->powerbase_voltage);
	mvprintw(i++, col, "velocity command:  %d\n", rmp->velocity_command);
	mvprintw(i++, col, "turn command:      %d\n", rmp->turn_command);
	//pthread_mutex_unlock(rmpGetMutex(dev));
}

int
main(int argc, char *argv[])
{
	struct RMP_DEV_STR **rmp;
	int i, ch, done = 0;
	int n1, n2;
	short v = 0, w = 0;
	int sending = 1;

	if (rmpOpenAll(&rmp) < 0) 
		exit(2);
	initscr();
	cbreak();
	timeout(0);
	noecho();
	while (!done) {
		if (rmp[0] != NULL) 
			if  ((n1 = rmpReadPackets(rmp[0])) < 0) {
				fprintf(stderr, "read error\n");
				break;
			}
		
		if (rmp[1] != NULL)
			if ((n2 = rmpReadPackets(rmp[1])) < 0) {
				fprintf(stderr, "read error\n");
				break;
			}
		mvprintw(0, 0, "v: %04d  w: %04d", v, w);
		if (rmp[0] != NULL && n1 > 0)
			showData(rmp[0], 1);
		if (rmp[1] != NULL && n2 > 0)
			showData(rmp[1], 41);
		mvprintw(33, 0, "%9lu %d", time(NULL), sending);
		ch = mvgetch(34, 0);
		switch (ch) {
		case ' ':
			v = 0;
			w = 0;
			break;
		case 'j':
			v -= 10;
			break;
		case 'k':
			v += 10;
			break;
		case 'h':
			w += 10;
			break;
		case 'l':
			w -= 10;
			break;
		case 'r':
			sending = 1;
			break;
		case 0x1b:
		case 'q':
			done = 1;
			break;
		}
		for (i = 0; i < 2; i++)
			if (rmp[i] != NULL && sending) {
				if (rmpSendSpeed(rmp[i], v, w) < 0)
					sending = FALSE;
			}
						
		refresh();
		
	}
	endwin();
	for (i = 0; i < 2; i++) {
		if (rmp[i] != NULL)
			rmpClose(rmp[i]);
	}
	return 0;
}


